using System;
using System.Runtime.Serialization;
using System.Threading;

namespace YAPS
{
    /*
     * YAPS - Yet Another Proxy Server: UDP Multicast to TCP Unicast Proxy (with VCR)
     * (C) Daniel Kirstenpfad 2006, HTTP portions (C) 2001 by Sam Pullara; ICON (C) 2003-2004 by Steven W. Smith
     * 
     * This is a small tool which allows you to proxy udp rtp multicast to tcp unicast (wihtout the rtp enclosure).
     * Beside this functionality YAPS implements a HTTP webserver and a VCR.
     * 
     * For updates and more information visit the YAPS website: http://www.technology-ninja.com
     * 
     */

    /// <summary>
    /// Yet Another Proxy Server: UDP Multicast to TCP Unicast Proxy and VCR
    /// </summary>
    class YAPSApp
	{
		[STAThread]
		static void Main(String[] args)
        {
            #region Logo
            ConsoleOutputLogger.WriteLine("Yet Another Proxy Server: UDP Multicast to TCP Unicast Proxy 1.0");
            ConsoleOutputLogger.WriteLine("(C) 2006 Daniel Kirstenpfad - http://www.technology-ninja.com");
            #endregion

            #region Data
            HttpServer httpServer;
            VCRScheduler vcrScheduler;
			int port = 8001;
			string docRoot = ".";
            httpServer = new HttpServer(docRoot, port);
            vcrScheduler = new VCRScheduler(httpServer);
            #endregion

            #region Testing
            // set up testing recording data
/*          Recording testRecording = new Recording();

            testRecording.Channel = "13";
            testRecording.StartsAt = DateTime.Now.AddHours(1);
            testRecording.EndsAt = testRecording.StartsAt.AddSeconds(30);

            testRecording.Recording_Filename = testRecording.Recording_ID.ToString();
            testRecording.Recording_Name = "Das perfekte Dinner";

            vcrScheduler.Recordings.Add(testRecording.Recording_ID, testRecording);

            testRecording = new Recording();

            testRecording.Channel = "20";
            testRecording.StartsAt = DateTime.Now.AddDays(1);
            testRecording.EndsAt = testRecording.StartsAt.AddSeconds(30);

            testRecording.Recording_Filename = testRecording.Recording_ID.ToString();
            testRecording.Recording_Name = "testaufnahme";

            vcrScheduler.Recordings.Add(testRecording.Recording_ID, testRecording);
*/
            #endregion

            #region Load the Settings...
            Settings Configuration = new Settings("YAPS.recordings.dat","YAPS.donerecordings.dat",vcrScheduler,httpServer);

            Configuration.LoadSettings();

            // tell the processors that we exist...
            vcrScheduler.Configuration = Configuration;
            httpServer.Configuration = Configuration;
            #endregion

            #region Starting up...
            #region HTTP Server
            ConsoleOutputLogger.WriteLine("Starting HTTP Server...");
			    Thread http_server_thread = new Thread(new ThreadStart(httpServer.listen));
			    http_server_thread.Start();
                #endregion

                #region VCR
                ConsoleOutputLogger.WriteLine("Starting VCR Scheduler...");
                Thread vcr_scheduler_thread = new Thread(new ThreadStart(vcrScheduler.RecordingScheduler));
                vcr_scheduler_thread.Start();
                #endregion
            #endregion
        }
	}
}
