using System;
using System.Collections;
using System.Text;

namespace YAPS
{
    /// <summary>
    /// Map the Channelnames to Numbers and vice versa
    /// </summary>
    public static class MapNames
    {
        private static Hashtable ChannelNames = new Hashtable();

        #region Add/Remove

        #region Add
        /// <summary>
        /// Adds a Number+Name Combination to the Mapping table
        /// </summary>
        /// <param name="Number">Channel-Number</param>
        /// <param name="Name">Channel-Name</param>
        /// <returns>true if successfully added, false if already existed/failed</returns>
        public static bool Add(String Number, String Name)
        {
            try
            {
                if (!ChannelNames.ContainsKey(Convert.ToInt32(Number)))
                {
                    ChannelNames.Add(Convert.ToInt32(Number), Name);
                    return true;
                }
                else 
                    return false;
            }
            catch(Exception e)
            {
                ConsoleOutputLogger.WriteLine("MapNames Add: " + e.Message);
                return false;
            }
        }
        /// <summary>
        /// Adds a Number+Name Combination to the Mapping table
        /// </summary>
        /// <param name="Number">Channel-Number</param>
        /// <param name="Name">Channel-Name</param>
        /// <returns>true if successfully added, false if already existed/failed</returns>
        public static bool Add(Int32 Number, String Name)
        {
            return Add(Convert.ToString(Number), Name);
        }
        #endregion

        #region Remove
        /// <summary>
        /// Remove a Name Mapping from the Mapping table
        /// </summary>
        /// <param name="Number">the Channel Number</param>
        /// <returns>true if successful, false if not</returns>
        public static bool Remove(Int32 Number)
        {
            try
            {
                ChannelNames.Remove(Number);
                return true;
            }
            catch (Exception e)
            {
                ConsoleOutputLogger.WriteLine("MapNames Remove: " + e.Message);
                return false;
            }
        }
        /// <summary>
        /// Remove a Name Mapping from the Mapping table
        /// </summary>
        /// <param name="Name">the Channel Name</param>
        /// <returns>true if successful, false if not</returns>
        public static bool Remove(String Name)
        {
            try
            {
                return Remove(Name2Number(Name));
            }
            catch (Exception e)
            {
                ConsoleOutputLogger.WriteLine("MapNames Remove: " + e.Message);
                return false;
            }
        }

        #endregion

        #endregion

        #region Mapping
        public static String Number2Name(int ChannelNumber)
        {
            return (String)ChannelNames[ChannelNumber];
        }
        public static Int32 Name2Number(String ChannelName)
        {
            if (ChannelNames.ContainsValue(ChannelName))
            {
                foreach (Int32 Key in ChannelNames.Keys)
                {
                    if ((string)ChannelNames[Key] == ChannelName)
                    {
                        return (Int32)Key;
                    }
                }
                return -1;
            }
            else
                return -1;
        }
        #endregion

        #region Listings
        /// <summary>
        /// Returns the Mapping Table
        /// </summary>
        /// <returns></returns>
        public static Hashtable GetMapTable()
        {
            return ChannelNames;
        }
        
        #endregion
    }
}
