using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

namespace YAPS
{
    /// <summary>
    /// each file request that leads to a .html file is passed through here
    /// </summary>
    public class TemplateProcessor
    {
        #region internal Data
        private VCRScheduler internal_vcrscheduler;
        private HttpProcessor internal_httpprocessor;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor of the Templace Processor
        /// </summary>
        /// <param name="vcrscheduler">the VCRScheduler...so we can get the </param>
        public TemplateProcessor(VCRScheduler vcrscheduler)
        {
            internal_vcrscheduler = vcrscheduler;
        }
        #endregion

        #region ProcessHTMLTemplate
        /// <summary>
        /// Processes the actual HTML Sourcecode
        /// </summary>
        /// <param name="template_filename"></param>
        /// <returns>the actual HTML code after the template parsing and keyword replaceing</returns>
        public String ProcessHTMLTemplate(String template_filename,String Querystring)
        {
            String Output_HTML_Code = "";

            #region read the template file
            using (StreamReader sr = File.OpenText(template_filename))
            {
                String buffer;

                while ((buffer = sr.ReadLine()) != null)
                {
                    Output_HTML_Code = Output_HTML_Code + buffer;
                    // read...
                }
                sr.Close();
            }
            #endregion

            #region replace the placeholders...

            // TODO: at the moment the placeholders must be lower case only to be detected - change that!

            #region %recording_listing%
            // first find the placeholder
            while (Output_HTML_Code.Contains("%recording_listing%"))
            {
                Output_HTML_Code = Output_HTML_Code.Replace("%recording_listing%", Template_Recording_Listing());
            }
            #endregion

            #region %querystring%
            // first find the placeholder
            while (Output_HTML_Code.Contains("%querystring%"))
            {
                Output_HTML_Code = Output_HTML_Code.Replace("%querystring%", Querystring);
            }
            #endregion

            #region %recorded_listing%
            // first find the placeholder
            while (Output_HTML_Code.Contains("%recorded_listing%"))
            {
                Output_HTML_Code = Output_HTML_Code.Replace("%recorded_listing%", Template_Recorded_Listing());
            }
            #endregion

            #region %console_output%
            // first find the placeholder
            while (Output_HTML_Code.Contains("%console_output%"))
            {
                Output_HTML_Code = Output_HTML_Code.Replace("%console_output%", Template_Console_Output());
            }
            #endregion


            #endregion

            return Output_HTML_Code;
        }
        #endregion

        #region HTML Code Generators

        // TODO: Sort the Recording Listings...

        #region RecordingListing
        /// <summary>
        /// this generates the Record Listing HTML Sourcecode
        /// </summary>
        /// <returns>Record Listing Sourcecode</returns>
        String Template_Recording_Listing()
        {
            StringBuilder Output = new StringBuilder();

            // BUG: when there's nothing currently recording the header of the table alone is shown...fix that

            // start the table
            Output.Append("<table width=\"90%\"><tr style=\"background-color: #93939a;\"><td style=\"width:50px\" align=\"center\">Status</td><td align=\"center\">Time</td><td align=\"center\">Sender</td><td align=\"center\">Name</td></tr>");

            if (internal_vcrscheduler.doneRecordings.Count > 0)
            {
                lock (internal_vcrscheduler.doneRecordings.SyncRoot)
                {
                    foreach (Recording recording_entry in internal_vcrscheduler.doneRecordings.Values)
                    {
                        if (recording_entry.CurrentlyRecording == true)
                        {
                            Output.Append("<tr style=\"background-color: #a29ca2;\"><td style=\"background-color:red;width:50px\"></td><td align=\"center\" style=\"width:150px\">");
                            // Date and Time
                            Output.Append(recording_entry.StartsAt.ToShortDateString());
                            Output.Append(" ");
                            Output.Append(recording_entry.StartsAt.ToShortTimeString());

                            // Channel
                            Output.Append("</td><td align=\"center\" style=\"width:90px\">");
                            Output.Append(recording_entry.Channel);

                            // Recording Name
                            Output.Append("</td><td align=\"center\">");
                            Output.Append(recording_entry.Recording_Name);
                            Output.Append("</td>");
                        }
                    }
                }

            }
            // just work when there is work to do...
            if (internal_vcrscheduler.Recordings.Count > 0)
            {
                lock (internal_vcrscheduler.Recordings.SyncRoot)
                {
                    foreach (Recording recording_entry in internal_vcrscheduler.Recordings.Values)
                    {
                        Output.Append("<tr style=\"background-color: #a29ca2;\"><td style=\"background-color:green;width:50px\"></td><td align=\"center\" style=\"width:150px\">");
                        // Date and Time
                        Output.Append(recording_entry.StartsAt.ToShortDateString());
                        Output.Append(" ");
                        Output.Append(recording_entry.StartsAt.ToShortTimeString());

                        // Channel
                        Output.Append("</td><td align=\"center\" style=\"width:90px\">");
                        Output.Append(recording_entry.Channel);

                        // Recording Name
                        Output.Append("</td><td align=\"center\">");
                        Output.Append(recording_entry.Recording_Name);
                        Output.Append("</td><td><a href=\"/vcr/deleterecording?id=");
                        Output.Append(Convert.ToBase64String(recording_entry.Recording_ID.ToByteArray()));
                        Output.Append("\">L&ouml;schen</a></td></tr>");
                    }
                }
            }

            Output.Append("</table>");

            if ((internal_vcrscheduler.Recordings.Count == 0) && (internal_vcrscheduler.doneRecordings.Count == 0))
            {
                Output.Remove(0, Output.Length);
                Output.Append("No Recordings...");
            }

            return Output.ToString();
        }
        #endregion

        #region RecordedListing
        /// <summary>
        /// this generates the Record Listing HTML Sourcecode
        /// </summary>
        /// <returns>Record Listing Sourcecode</returns>
        String Template_Recorded_Listing()
        {
            StringBuilder Output = new StringBuilder();

            // start the table
            Output.Append("<table width=\"90%\"><tr style=\"background-color: #93939a;\"><td align=\"center\">Time</td><td align=\"center\">Sender</td><td align=\"center\">Name</td></tr>");

            if (internal_vcrscheduler.doneRecordings.Count > 0)
            {
                lock (internal_vcrscheduler.doneRecordings.SyncRoot)
                {
                    foreach (Recording recording_entry in internal_vcrscheduler.doneRecordings.Values)
                    {
                        Output.Append("<tr style=\"background-color: #a29ca2;\"><td align=\"center\" style=\"width:150px\">");
                        // Date and Time
                        Output.Append(recording_entry.StartsAt.ToShortDateString());
                        Output.Append(" ");
                        Output.Append(recording_entry.StartsAt.ToShortTimeString());

                        // Channel
                        Output.Append("</td><td align=\"center\" style=\"width:90px\">");
                        Output.Append(recording_entry.Channel);

                        // Recording Name
                        Output.Append("</td><td align=\"center\">");
                        Output.Append(recording_entry.Recording_Name);
                        Output.Append("</td>");
                        // TODO: make it configurable...Internationalization
                        // TODO: make the servername configurable...
                        Output.Append("<td><a href=\"http://bubblegum:8001/");
                        Output.Append(recording_entry.Recording_ID.ToString());
                        Output.Append("\">Ansehen</a></td>");

                        Output.Append("<td><a href=\"/vcr/RemoveRecordingFile?id=");
                        Output.Append(recording_entry.Recording_ID.ToString());
                        Output.Append("\">L&ouml;schen</a></td>");
                        
                    }
                }
            }

            Output.Append("</table>");

            if ((internal_vcrscheduler.Recordings.Count == 0) && (internal_vcrscheduler.doneRecordings.Count == 0))
            {
                Output.Remove(0, Output.Length);
                Output.Append("No Recordings...");
            }

            return Output.ToString();
        }
        #endregion

        #region ConsoleOutput
        /// <summary>
        /// this generates the Record Listing HTML Sourcecode
        /// </summary>
        /// <returns>Record Listing Sourcecode</returns>
        String Template_Console_Output()
        {
            StringBuilder Output = new StringBuilder();

            String[] ConsoleOutput = ConsoleOutputLogger.GetLoggedLines();

            Output.Append("<pre>");

            foreach (String line in ConsoleOutput)
            {
                Output.Append(line + "\n");
            }
            Output.Append("</pre>");
            return Output.ToString();
        }
        #endregion


        #endregion
    }
}
